from __future__ import annotations

import os

from ews_core_config.config import read_settings


def test_config():
    settings = read_settings(create_if_inexistent=True).get_settings()
    envs = {k: v for k, v in os.environ.items() if k.startswith("EWS_")}
    assert envs


def test_user_file_read():
    settings = read_settings(create_if_inexistent=True)
    assert settings.sharepoint_username != ""



from __future__ import annotations

import logging
import os
import socket
import subprocess
from copy import deepcopy
from pathlib import Path

import tomlkit

logging.getLogger("simple_toml_configurator").setLevel(logging.ERROR)
logging.getLogger("Configuration").setLevel(logging.ERROR)
from simple_toml_configurator import Configuration  # noqa: E402

logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)

hostname = socket.gethostname().upper()

name = email = gh_username = ""
try:
    p = subprocess.run(
        "git config --get user.name",  # noqa: S607
        shell=True,  # noqa: S602
        capture_output=True,
        text=True,
        check=False,
    )
    if not p.returncode:
        name = p.stdout.strip()
    p = subprocess.run(
        "git config --get user.email",  # noqa: S607
        shell=True,  # noqa: S602
        capture_output=True,
        text=True,
        check=False,
    )
    if not p.returncode:
        email = p.stdout.strip()
        gh_username = "ews-" + email.split("@")[0].replace(".", "")
except Exception as e:
    raise e
finally:
    pass


DEFAULT_CONFIG = {
    "sharepoint": {
        "server": "ewsconsulting.sharepoint.com",
        "site": "https://ewsconsulting.sharepoint.com/sites/teams_mb/",
        "username": email,
        "password": "",
    },
    "github": {
        "username": gh_username,
        "token": "",
    },
    "ewstools": {
        "url": "https://mb.ews.tools",
        "host": "smufcm.ews.local",
        "port": "52812",
        "username": "mb_upload",
        "password": "",
    },
    "paths": {
        "mount_point": "/mnt/ews_drives",
        "drives": "f;p;r",
        "mappings": "/smuffile001/daten$:f;/smuffile001/qm$:p;/smuffile001/ROHDATEN$:r",
    },
    "wheelhouse": {
        "local": {
            "devpi": {
                "hostname": hostname,
                "port": 8051,
                "index": "root/private",
                "index-url": "root/all/+simple",
            },
            "pyserver": {
                "hostname": hostname,
                "port": 8050,
                "index": "",
                "index-url": "simple",
            },
        },
        "global": {
            "pyserver": {
                "hostname": "WMUFS100",
                "port": 3140,
                "index": "",
                "index-url": "simple",
            },
            "devpi": {
                "hostname": "WMUFS100",
                "port": 3141,
                "index": "root/private",
                "index-url": "root/all/+simple",
            },
        },
    },
}


def recursive_pop(val, parent=None, key=None):  # noqa: C901
    has_parent = parent and key
    if isinstance(val, dict):
        if not val and has_parent:
            parent.pop(key)
            return None
        new_vals = deepcopy(val)
        keys = list(new_vals.keys())
        for k in keys:
            recursive_pop(new_vals[k], parent=new_vals, key=k)
        if has_parent:
            if new_vals:
                parent[key] = new_vals
            else:
                parent.pop(key)
                return None
        return new_vals
    elif isinstance(val, list):
        if not val and has_parent:
            parent.pop(key)
            return parent
        new_vals = []
        for _, v in enumerate(val):
            new_v = recursive_pop(v)
            if new_v is not None:
                new_vals.append(new_v)

        if has_parent:
            if new_vals:
                parent[key] = new_vals
            else:
                parent.pop(key)
                return None
        return new_vals
    elif isinstance(val, str):
        v = val.strip()
        if not v and has_parent:
            parent.pop(key)
        return None
    elif val is None:
        if has_parent:
            parent.pop(key)
        return None
    else:
        return val


def merge(source, destination):
    """
    run me with nosetests --with-doctest file.py

    >>> a = { 'first' : { 'all_rows' : { 'pass' : 'dog', 'number' : '1' } } }
    >>> b = { 'first' : { 'all_rows' : { 'fail' : 'cat', 'number' : '5' } } }
    >>> merge(b, a) == { 'first' : { 'all_rows' : { 'pass' : 'dog', 'fail' : 'cat', 'number' : '5' } } }
    True
    """
    for key, value in source.items():
        if isinstance(value, dict):
            # get node or create one
            node = destination.setdefault(key, {})
            merge(value, node)
        else:
            destination[key] = value

    return destination


class _EWSSettings(Configuration):
    def __init__(self, create_if_inexistent: bool = True, update: bool = True) -> None:
        config_path = os.environ.get("EWS_CONFIG_PATH", Path.home() / ".config" / "ews")
        config_file_name = os.environ.get("EWS_CONFIG_FILENAME", ".ews_config")
        filename = Path(config_path) / f"{config_file_name}.toml"
        existing = {}

        if filename.is_file():
            logger.debug("Reading defaults from existing file")
            existing = tomlkit.loads(filename.read_bytes())

        if not existing:
            if create_if_inexistent:
                logger.warn("Creating new file")  # pragma: no cover
                existing = deepcopy(DEFAULT_CONFIG)
        else:
            if update:
                existing = recursive_pop(existing)
            existing = merge(existing, DEFAULT_CONFIG)

        if filename.is_file():
            filename.unlink()
        super().__init__(
            config_path=config_path,
            config_file_name=config_file_name,
            defaults=existing,
            env_prefix="EWS",
        )


def read_settings(create_if_inexistent: bool = True, update: bool = True) -> _EWSSettings:
    return _EWSSettings(create_if_inexistent=create_if_inexistent, update=update)


__all__ = ("_EWSSettings", "read_settings")



from __future__ import annotations

import io
import os
from pprint import pprint

import click

from ews_core_config.version import VERSION


@click.group()
@click.version_option(VERSION, "-v", "--version", is_flag=True, help="Show the version")
def cli():
    pass


def show_dict_as_text(d: dict, min_pad: int = 20, **kwargs):
    if not d:
        return
    max_pad = max(map(len, d.keys())) + 3
    max_pad = max(min_pad, max_pad)
    for k, v in d.items():
        txt = f"{k.ljust(max_pad)}: {v}"
        click.secho(txt, **kwargs)


def pprint_dict(d: dict, **kwargs):
    if not d:
        return
    stream = io.StringIO()
    pprint(d, stream=stream)
    stream = stream.getvalue()
    click.secho(stream, **kwargs)


@cli.command(name="show", help="Show the config attributes and values")
@click.option("-t/-nt", "--text/--no-text", is_flag=True, default=True, show_default=True)
@click.option("-l/-nl", "--location/--no-location", is_flag=True, default=True, show_default=True)
def show(text: bool = True, location: bool = True):
    from ews_core_config.config import _EWSSettings

    settings = _EWSSettings(create_if_inexistent=False)
    cfg_filename = settings._full_config_path
    if location:
        click.secho(f"Location: {cfg_filename!s}\n", fg="yellow", bold=True)

    nice = settings.get_settings()
    if text:
        show_dict_as_text(nice, fg="blue")
    else:
        pprint_dict(nice, fg="blue")


@cli.command(name="env", help="Show the set environment variables")
@click.option("-t/-nt", "--text/--no-text", is_flag=True, default=True, show_default=True)
@click.option("-l/-nl", "--location/--no-location", is_flag=True, default=True, show_default=True)
def env(text: bool = True, location: bool = True):
    from ews_core_config.config import _EWSSettings

    settings = _EWSSettings(create_if_inexistent=False)

    cfg_filename = settings._full_config_path
    if location:
        click.secho(f"Location: {cfg_filename!s}\n", fg="yellow", bold=True)
    env_prefix = settings.env_prefix
    env_vars = {k: var for k, var in os.environ.items() if k.startswith(f"{env_prefix}_")}
    if text:
        show_dict_as_text(env_vars, fg="blue")
    else:
        pprint_dict(env_vars, fg="blue")


@cli.command(name="path", help="Show the path of the config file")
def _path():
    from ews_core_config.config import _EWSSettings

    cfg_filename = _EWSSettings(create_if_inexistent=False)._full_config_path
    click.echo(f"{cfg_filename!s}")


@cli.command(name="init", help="Create a new config file if it does not exist")
def _init():
    from ews_core_config.config import _EWSSettings

    cfg_filename = _EWSSettings(create_if_inexistent=True)._full_config_path
    click.echo(f"{cfg_filename!s}")


@cli.command(name="update", help="Update a config file if it does not exist")
def _update():
    from ews_core_config.config import _EWSSettings

    cfg = _EWSSettings(create_if_inexistent=True, update=True)
    cfg_filename = cfg._full_config_path
    click.echo(f"{cfg_filename!s}")
